/*****************************************************************************************************************************
--
--    File Name    : write_channel1_top.v  

--    Description  : This module is the write channel 1 top-level
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module write_channel1_top #(parameter g_AXI_AWIDTH = 32,
                            parameter g_AXI_DWIDTH = 64,
                            parameter g_AXI_BUFF_AWIDTH = 12,
                            parameter g_HORIZONTAL_RESOLUTION = 1280,
                            parameter g_WR_CHANNEL1_VIDEO_DATA_WIDTH = 24,
                            parameter g_WR_CHANNEL1_BUFFER_LINE_STORAGE = 1)
                          (input wire clock,
                           input wire rclock,
                           input wire nreset,
                           input wire arbitr_ack,
                           input wire axi_data_ren,
                           input wire axi_tr_done,
                           input wire[(g_AXI_BUFF_AWIDTH + 3) - 1 : 0] bytes_to_write_1,
                           input wire[(g_AXI_AWIDTH - 1) : 0] start_write_addr_1,
                           input wire[(g_WR_CHANNEL1_VIDEO_DATA_WIDTH - 1) : 0] video_wdata_1,
                           input wire wdata_valid_1,
                           input wire wr_req_1,
                           output wire[3 : 0] axi_burst_len,
                           output wire[1 : 0] axi_burst_typ,
                           output wire[(g_AXI_AWIDTH - 1) : 0] axi_daddr,
                           output wire axi_start,
                           output wire[1 : 0] axi_tr_dir,
                           output wire[2 : 0] axi_tr_size,
                           output wire[(g_AXI_DWIDTH - 1) : 0] axi_wr_data_o,
                           output wire master_wr_req,
                           output wire wr_ack_1,
                           output wire wr_burst_split,
                           output wire wr_done_1);
  parameter i01I = ((g_HORIZONTAL_RESOLUTION * g_WR_CHANNEL1_VIDEO_DATA_WIDTH * g_WR_CHANNEL1_BUFFER_LINE_STORAGE) / g_AXI_DWIDTH);
  wire[(g_AXI_DWIDTH - 1) : 0] oO0I;
  wire[3 : 0] iO0I;
  wire[1 : 0] OI0I;
  wire[(g_AXI_AWIDTH - 1) : 0] O11I;
  wire[(g_AXI_AWIDTH - 1) : 0] I11I;
  wire[(g_AXI_BUFF_AWIDTH - 1) : 0] l11I;
  wire o11I;
  wire Il0I;
  wire[1 : 0] ll0I;
  wire[2 : 0] ol0I;
  wire[(g_AXI_DWIDTH - 1) : 0] i11I;
  wire[4 : 0] Oo1I;
  wire[(g_AXI_AWIDTH - 1) : 0] Io1I;
  wire lo1I;
  wire oo1I;
  wire io1I;
  wire[(g_AXI_DWIDTH - 1) : 0] Oi1I;
  wire[(g_AXI_BUFF_AWIDTH - 1) : 0] Ii1I;
  wire li1I;
  wire o1OI;
  wire oi1I;
  wire OoOI;
  wire IoII;
  wire loII;
  wire ii1I;
  wire Io0I;
  wire OOoI;
  wire[2 : 0] oi0I;
  wire[(g_AXI_DWIDTH - 1) : 0] IOoI;
  wire[1 : 0] Oi0I;
  wire[3 : 0] io0I;
  wire[1 : 0] oo0I;
  wire lOoI;
  assign IoII = OoOI;
  assign wr_done_1 = IoII;
  assign loII = o1OI;
  assign wr_ack_1 = loII;
  assign I11I = O11I;
  assign axi_daddr = I11I;
  assign ii1I = oi1I;
  assign wr_burst_split = ii1I;
  assign Io0I = Il0I;
  assign axi_start = Io0I;
  assign OOoI = io1I;
  assign master_wr_req = OOoI;
  assign oi0I = ol0I;
  assign axi_tr_size = oi0I;
  assign IOoI = i11I;
  assign axi_wr_data_o = IOoI;
  assign Oi0I = ll0I;
  assign axi_tr_dir = Oi0I;
  assign io0I = iO0I;
  assign axi_burst_len = io0I;
  assign oo0I = OI0I;
  assign axi_burst_typ = oo0I;
  axi_buffer #(.g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH), .AXI_BUFF_DEPTH(i01I), .g_AXI_DWIDTH(g_AXI_DWIDTH)) ii0I(.wr_data_i(Oi1I),
                                                                                                               .rd_addr(l11I),
                                                                                                               .wr_addr(Ii1I),
                                                                                                               .we(li1I),
                                                                                                               .wclock(clock),
                                                                                                               .rclock(rclock),
                                                                                                               .rd_data_o(oO0I));
  AXI_master_write_ch #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_DWIDTH(g_AXI_DWIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) oOoI(.clock(clock),
                                                                                                                              .nreset(nreset),
                                                                                                                              .axi_wr_ddr_addr(Io1I),
                                                                                                                              .burst_length(Oo1I),
                                                                                                                              .start_axi_tr(lo1I),
                                                                                                                              .axi_tr_done(axi_tr_done),
                                                                                                                              .arbitr_ack(arbitr_ack),
                                                                                                                              .axi_data_ren(axi_data_ren),
                                                                                                                              .axi_buff_datain(oO0I),
                                                                                                                              .last_burst(lOoI),
                                                                                                                              .transf_compl(o11I),
                                                                                                                              .axi_daddr(O11I),
                                                                                                                              .axi_tr_size(ol0I),
                                                                                                                              .axi_burst_typ(OI0I),
                                                                                                                              .axi_burst_len(iO0I),
                                                                                                                              .axi_tr_dir(ll0I),
                                                                                                                              .axi_start(Il0I),
                                                                                                                              .master_wr_req(io1I),
                                                                                                                              .axi_wr_data_o(i11I),
                                                                                                                              .axi_buff_raddr(l11I));
  ddr_write_contrl #(.g_AXI_AWIDTH(g_AXI_AWIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) iOoI(.clock(clock),
                                                                                              .nreset(nreset),
                                                                                              .wr_req_i(wr_req_1),
                                                                                              .ddr_write_start_address_i(start_write_addr_1),
                                                                                              .bytes_to_write_i(bytes_to_write_1),
                                                                                              .axi_tr_done(o11I),
                                                                                              .wr_ack_o(o1OI),
                                                                                              .wr_done_o(OoOI),
                                                                                              .axi_wr_ddr_addr(Io1I),
                                                                                              .axi_burst_len(Oo1I),
                                                                                              .start_axi_tr(lo1I),
                                                                                              .start_pack(oo1I),
                                                                                              .last_burst_o(lOoI),
                                                                                              .wr_burst_split(oi1I));
  generate
    if (g_WR_CHANNEL1_VIDEO_DATA_WIDTH == 24)
      begin
        pack_24_64 #(.g_AXI_DWIDTH(g_AXI_DWIDTH), .g_WR_CHANNEL_VIDEO_DATA_WIDTH(g_WR_CHANNEL1_VIDEO_DATA_WIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) OIoI(.clock(clock),
                                                                                                                                                              .nreset(nreset),
                                                                                                                                                              .start_pack(oo1I),
                                                                                                                                                              .video_data_in(video_wdata_1),
                                                                                                                                                              .data_valid(wdata_valid_1),
                                                                                                                                                              .buff_data_o(Oi1I),
                                                                                                                                                              .buff_waddr(Ii1I),
                                                                                                                                                              .buff_we(li1I));
      end
    else
      if (g_WR_CHANNEL1_VIDEO_DATA_WIDTH == 32)
        begin
          pack_32_64 #(.g_AXI_DWIDTH(g_AXI_DWIDTH), .g_WR_CHANNEL_VIDEO_DATA_WIDTH(g_WR_CHANNEL1_VIDEO_DATA_WIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) IIoI(.clock(clock),
                                                                                                                                                                .nreset(nreset),
                                                                                                                                                                .start_pack(oo1I),
                                                                                                                                                                .video_data_in(video_wdata_1),
                                                                                                                                                                .data_valid(wdata_valid_1),
                                                                                                                                                                .buff_data_o(Oi1I),
                                                                                                                                                                .buff_waddr(Ii1I),
                                                                                                                                                                .buff_we(li1I));
        end
      else
        if (g_WR_CHANNEL1_VIDEO_DATA_WIDTH == 8)
          begin
            pack_8_64 #(.g_AXI_DWIDTH(g_AXI_DWIDTH), .g_WR_CHANNEL_VIDEO_DATA_WIDTH(g_WR_CHANNEL1_VIDEO_DATA_WIDTH), .g_AXI_BUFF_AWIDTH(g_AXI_BUFF_AWIDTH)) lIoI(.clock(clock),
                                                                                                                                                                 .nreset(nreset),
                                                                                                                                                                 .start_pack(oo1I),
                                                                                                                                                                 .video_data_in(video_wdata_1),
                                                                                                                                                                 .data_valid(wdata_valid_1),
                                                                                                                                                                 .buff_data_o(Oi1I),
                                                                                                                                                                 .buff_waddr(Ii1I),
                                                                                                                                                                 .buff_we(li1I));
          end
  endgenerate
endmodule
