/*****************************************************************************************************************************
--
--    File Name    : display_controller.v

--    Description  : Top-level HDL for display controller
--                   

-- Targeted device : Microsemi-SoC
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/

`timescale 1ns/1ps
module display_controller #(parameter g_DDR_AXI_AWIDTH = 32,
                            parameter g_INPUT_X_W_RES_WIDTH = 12,
                            parameter g_INPUT_Y_H_RES_WIDTH = 12,
                            parameter g_VIDEO_FIFO_AWIDTH = 13,
                            parameter g_INPUT_VIDEO_DATA_BIT_WIDTH = 24,
                            parameter g_DEPTH_OF_VIDEO_PIXEL_FROM_DDR = 1,
                            parameter g_HORZ_SYNC_PULSE_POLARITY = 1,
                            parameter g_VERT_SYNC_PULSE_POLARITY = 1,
                            parameter g_INITIAL_LINES_TO_BUFFER = 6,
                            parameter g_SUBSEQUENT_LINES_TO_BUFFER = 1,
                            parameter g_HALF_EMPTY_THRESHOLD = 3840)
                          (input wire RESET_N_I,
                           input wire SYS_CLOCK_I,
                           input wire VIDEO_CLOCK_I,
                           input wire display_control_enable_i,
                           input wire[(g_INPUT_VIDEO_DATA_BIT_WIDTH - 1) : 0] video_datain_i,
                           input wire data_valid_i,
                           input wire[(g_DDR_AXI_AWIDTH - 1) : 0] frame_buff_addr_i,
                           input wire frame_ddr_addr_valid_i,
                           input wire read_ack_i,
                           input wire read_done_i,
                           input wire[(g_INPUT_X_W_RES_WIDTH - 1) : 0] tot_horz_pixels_i,
                           input wire[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] tot_vert_lines_i,
                           input wire[31 : 0] video_resolution_i,
                           input wire[31 : 0] ddr_rd_video_resolution_i,
                           input wire[31 : 0] back_porch_width_i,
                           input wire[31 : 0] front_porch_width_i,
                           input wire[(g_INPUT_X_W_RES_WIDTH - 1) : 0] hsync_width_i,
                           input wire[(g_INPUT_Y_H_RES_WIDTH - 1) : 0] vsync_width_i,
                           output wire disp_cont_busy_o,
                           output wire disp_cont_done_o,
                           output wire[(g_DDR_AXI_AWIDTH - 1) : 0] frame_buff_und_disp_o,
                           output wire[15 : 0] bytes_to_read_o,
                           output wire[(g_DDR_AXI_AWIDTH - 1) : 0] start_read_addr_o,
                           output wire read_req_o,
                           output wire[23 : 0] rgb_out_o,
                           output wire data_enable_o,
                           output wire horz_sync_o,
                           output wire vert_sync_o,
                           output wire hactive_o,
                           output wire vactive_o,
                           output wire dg_fifo_empty,
                           output wire dg_fifo_full,
                           output wire dg_fifo_hemtpy);
  wire[15 : 0] IOl;
  wire lOl;
  wire oOl;
  wire iOl;
  wire[(g_DDR_AXI_AWIDTH - 1) : 0] OIl;
  wire IIl;
  wire lIl;
  wire[23 : 0] oIl;
  wire[(g_DDR_AXI_AWIDTH - 1) : 0] iIl;
  wire Oll;
  wire[23 : 0] Ill;
  wire lll;
  wire oll;
  wire ill;
  wire O0l;
  wire[(g_DDR_AXI_AWIDTH - 1) : 0] I0l;
  wire[15 : 0] l0l;
  wire[(g_DDR_AXI_AWIDTH - 1) : 0] o0l;
  wire i0l;
  wire[23 : 0] O1l;
  wire I1l;
  wire l1l;
  wire o1l;
  wire i1l;
  wire[(g_DDR_AXI_AWIDTH - 1) : 0] Ool;
  assign O0l = lIl;
  assign read_req_o = O0l;
  assign I0l = OIl;
  assign frame_buff_und_disp_o[31 : 0] = I0l;
  assign l0l = IOl;
  assign bytes_to_read_o[15 : 0] = l0l;
  assign o0l = iIl;
  assign start_read_addr_o[31 : 0] = o0l;
  assign i0l = Oll;
  assign vert_sync_o = i0l;
  assign O1l = oIl;
  assign rgb_out_o[23 : 0] = O1l;
  assign I1l = lOl;
  assign data_enable_o = I1l;
  assign l1l = IIl;
  assign horz_sync_o = l1l;
  assign o1l = iOl;
  assign disp_cont_done_o = o1l;
  assign i1l = oOl;
  assign disp_cont_busy_o = i1l;
  assign dg_fifo_hemtpy = lll;
  ddr_read_controller #(.g_DDR_AXI_AWIDTH(g_DDR_AXI_AWIDTH), .g_INPUT_X_W_RES_WIDTH(g_INPUT_X_W_RES_WIDTH), .g_INPUT_Y_H_RES_WIDTH(g_INPUT_Y_H_RES_WIDTH), .g_VIDEO_PIXEL_FROM_DDR_DEPTH(g_DEPTH_OF_VIDEO_PIXEL_FROM_DDR), .g_INITIAL_LINES_TO_BUFFER(g_INITIAL_LINES_TO_BUFFER), .g_SUBSEQUENT_LINES_TO_BUFFER(g_SUBSEQUENT_LINES_TO_BUFFER)) Iol(.RESET_N_I(RESET_N_I),
                                                                                                                                                                                                                                                                                                                                                   .SYS_CLOCK_I(SYS_CLOCK_I),
                                                                                                                                                                                                                                                                                                                                                   .display_control_enable_i(display_control_enable_i),
                                                                                                                                                                                                                                                                                                                                                   .frame_buff_addr_i(Ool),
                                                                                                                                                                                                                                                                                                                                                   .frame_ddr_addr_valid_i(frame_ddr_addr_valid_i),
                                                                                                                                                                                                                                                                                                                                                   .video_resolution_i(ddr_rd_video_resolution_i),
                                                                                                                                                                                                                                                                                                                                                   .start_ddr_read(ill),
                                                                                                                                                                                                                                                                                                                                                   .read_ack_i(read_ack_i),
                                                                                                                                                                                                                                                                                                                                                   .read_done_i(read_done_i),
                                                                                                                                                                                                                                                                                                                                                   .fifo_half_empty(lll),
                                                                                                                                                                                                                                                                                                                                                   .frame_complete(iOl),
                                                                                                                                                                                                                                                                                                                                                   .frame_buff_und_disp_o(OIl),
                                                                                                                                                                                                                                                                                                                                                   .read_req_o(lIl),
                                                                                                                                                                                                                                                                                                                                                   .start_read_addr_o(iIl),
                                                                                                                                                                                                                                                                                                                                                   .bytes_to_read_o(IOl));
  video_fifo #(.g_VIDEO_FIFO_AWIDTH(g_VIDEO_FIFO_AWIDTH), .g_INPUT_VIDEO_DATA_BIT_WIDTH(g_INPUT_VIDEO_DATA_BIT_WIDTH), .g_HALF_EMPTY_THRESHOLD(g_HALF_EMPTY_THRESHOLD)) lol(.wclock_i(SYS_CLOCK_I),
                                                                                                                                                                            .wresetn_i(RESET_N_I),
                                                                                                                                                                            .wen_i(data_valid_i),
                                                                                                                                                                            .wdata_i(video_datain_i),
                                                                                                                                                                            .rclock_i(VIDEO_CLOCK_I),
                                                                                                                                                                            .rresetn_i(RESET_N_I),
                                                                                                                                                                            .ren_i(oll),
                                                                                                                                                                            .wfull_o(dg_fifo_full),
                                                                                                                                                                            .wafull_o(),
                                                                                                                                                                            .wdata_count_o(),
                                                                                                                                                                            .rdata_o(Ill),
                                                                                                                                                                            .rempty_o(dg_fifo_empty),
                                                                                                                                                                            .raempty_o(),
                                                                                                                                                                            .rhempty_o(lll),
                                                                                                                                                                            .rdata_count_o());
  video_timing_generator #(.g_INPUT_X_W_RES_WIDTH(g_INPUT_X_W_RES_WIDTH), .g_INPUT_Y_H_RES_WIDTH(g_INPUT_Y_H_RES_WIDTH), .g_INPUT_VIDEO_DATA_BIT_WIDTH(g_INPUT_VIDEO_DATA_BIT_WIDTH), .g_HORZ_SYNC_PULSE_POLARITY(g_HORZ_SYNC_PULSE_POLARITY), .g_VERT_SYNC_PULSE_POLARITY(g_VERT_SYNC_PULSE_POLARITY)) ool(.RESET_N_I(RESET_N_I),
                                                                                                                                                                                                                                                                                                            .VIDEO_CLOCK_I(VIDEO_CLOCK_I),
                                                                                                                                                                                                                                                                                                            .display_control_enable_i(display_control_enable_i),
                                                                                                                                                                                                                                                                                                            .tot_horz_pixels_i(tot_horz_pixels_i),
                                                                                                                                                                                                                                                                                                            .tot_vert_lines_i(tot_vert_lines_i),
                                                                                                                                                                                                                                                                                                            .video_resolution_i(video_resolution_i),
                                                                                                                                                                                                                                                                                                            .back_porch_width_i(back_porch_width_i),
                                                                                                                                                                                                                                                                                                            .front_porch_width_i(front_porch_width_i),
                                                                                                                                                                                                                                                                                                            .hsync_width_i(hsync_width_i),
                                                                                                                                                                                                                                                                                                            .vsync_width_i(vsync_width_i),
                                                                                                                                                                                                                                                                                                            .video_data_in(Ill),
                                                                                                                                                                                                                                                                                                            .disp_cont_busy_o(oOl),
                                                                                                                                                                                                                                                                                                            .disp_cont_done_o(iOl),
                                                                                                                                                                                                                                                                                                            .start_ddr_read(ill),
                                                                                                                                                                                                                                                                                                            .fifo_re(oll),
                                                                                                                                                                                                                                                                                                            .rgb_out_o(oIl),
                                                                                                                                                                                                                                                                                                            .data_enable_o(lOl),
                                                                                                                                                                                                                                                                                                            .horz_sync_o(IIl),
                                                                                                                                                                                                                                                                                                            .vert_sync_o(Oll),
                                                                                                                                                                                                                                                                                                            .hactive_o(hactive_o),
                                                                                                                                                                                                                                                                                                            .vactive_o(vactive_o));
  bus_cdc_synchornizer #(.g_BUS_WIDTH(g_DDR_AXI_AWIDTH)) iol(.DEST_CLOCK_I(SYS_CLOCK_I),
                                                             .input_bus_i(frame_buff_addr_i),
                                                             .output_bus_o(Ool));
endmodule
